@@ -1,19 +1,19 @@
 import asyncio
 import json
 import logging
-from typing import Optional, Union
+from typing import Optional, Union, Dict, Any
 
 import aiohttp
 import requests
 
 
-def add_extra_properties_to_schema(schema, appsettings_path):
+def add_extra_properties_to_schema(schema: Dict[str, Any], appsettings_path: str) -> Dict[str, Any]:
     # Load the appsettings.json file
     with open(appsettings_path, 'r') as appsettings_file:
         appsettings = json.load(appsettings_file)
 
     # Initialize the extra properties dictionary
-    extra_properties = {}
+    extra_properties: Dict[str, Any] = {}
 
     # Mapping of Python types to JSON schema types
     type_mapping = {
@@ -46,9 +46,9 @@ class SingletonMeta(type):
     """
     This is a thread-safe implementation of Singleton.
     """
-    _instances = {}
+    _instances: Dict[type, Any] = {}
 
-    def __call__(cls, *args, **kwargs):
+    def __call__(cls, *args: Any, **kwargs: Any) -> Any:
         if cls not in cls._instances:
             instance = super().__call__(*args, **kwargs)
             cls._instances[cls] = instance
@@ -56,13 +56,24 @@ class SingletonMeta(type):
 
 
 class GoogleChatSyncMessenger(metaclass=SingletonMeta):
-    def __init__(self, service_name: str, g_chat_webhook: str, environ: str, log_enabled: Optional[list] = None,
-                 logger: Optional[logging.Logger] = None):
+    def __init__(
+            self,
+            service_name: str,
+            g_chat_webhook: str,
+            g_chat_error_webhook: str,
+            environ: str,
+            log_enabled: Optional[list[str]] = None,
+            logger: Optional[logging.Logger] = None
+    ) -> None:
         if log_enabled is None:
             log_enabled = ['production']
 
         self.service_name = service_name
         self.g_chat_webhook = g_chat_webhook
+        if g_chat_error_webhook:
+            self.g_chat_error_webhook = g_chat_error_webhook
+        else:
+            self.g_chat_error_webhook = g_chat_webhook
         self.environ = environ
         self.log_enabled = log_enabled
         self.logger = logger
@@ -71,7 +82,7 @@ class GoogleChatSyncMessenger(metaclass=SingletonMeta):
             0: '#FF0000',  # Red
         }
 
-    def send_message(self, text_message: str, message_type: int = None):
+    def send_message(self, text_message: str, message_type: int = None) -> None:
         """Send message to Google Chat synchronously."""
         try:
             message_headers = {'Content-Type': 'application/json; charset=UTF-8'}
@@ -82,7 +93,10 @@ class GoogleChatSyncMessenger(metaclass=SingletonMeta):
 
             # Determine color based on message type
             color = self.color_dict.get(message_type, '#000000')  # Default to black if unknown type
-
+            if message_type == 0:
+                url = self.g_chat_error_webhook
+            else:
+                url = self.g_chat_webhook
             bot_message = {
                 "cards": [
                     {
@@ -106,25 +120,37 @@ class GoogleChatSyncMessenger(metaclass=SingletonMeta):
             }
 
             response = requests.post(
-                url=self.g_chat_webhook,
+                url=url,
                 headers=message_headers,
                 data=json.dumps(bot_message),
             )
             response.raise_for_status()  # Raise an HTTPError if the HTTP request returned an unsuccessful status code
         except Exception as e:
-            self.logger.error('Error in webhookApi', exc_info=True)
+            if self.logger:
+                self.logger.error('Error in webhookApi', exc_info=True)
 
 
 class GoogleChatAsyncMessenger(metaclass=SingletonMeta):
-    def __init__(self, service_name: str, g_chat_webhook: str, environ: str, log_enabled: Optional[list] = None,
-                 logger: Optional[logging.Logger] = None, max_queue_size: int = 100):
+    def __init__(
+            self,
+            service_name: str,
+            g_chat_webhook: str,
+            g_chat_error_webhook: str,
+            environ: str,
+            log_enabled: Optional[list[str]] = None,
+            logger: Optional[logging.Logger] = None,
+            max_queue_size: int = 100
+    ) -> None:
         if log_enabled is None:
             log_enabled = ['PRODUCTION']
-        if logger is None:
-            logger = logging.getLogger(__name__)
+        if g_chat_error_webhook:
+            self.g_chat_error_webhook = g_chat_error_webhook
+        else:
+            self.g_chat_error_webhook = g_chat_webhook
 
         self.service_name = service_name
         self.g_chat_webhook = g_chat_webhook
+
         self.environ = environ
         self.log_enabled = log_enabled
         self.logger = logger
@@ -132,24 +158,24 @@ class GoogleChatAsyncMessenger(metaclass=SingletonMeta):
             1: '#008000',  # Green
             0: '#FF0000',  # Red
         }
-        self.log_queue = asyncio.Queue(max_queue_size)
-        self.queue_task = None
+        self.log_queue: asyncio.Queue[tuple[str, int]] = asyncio.Queue(max_queue_size)
+        self.queue_task: Optional[asyncio.Task] = None
 
-    def _ensure_queue_processing(self):
+    def _ensure_queue_processing(self) -> None:
         if not self.queue_task or self.queue_task.done():
             loop = asyncio.get_event_loop()
             self.queue_task = loop.create_task(self._process_log_queue())
 
-    async def _enqueue_message(self, text_message: str, message_type: int):
+    async def _enqueue_message(self, text_message: str, message_type: int) -> None:
         await self.log_queue.put((text_message, message_type))
 
-    async def _process_log_queue(self):
+    async def _process_log_queue(self) -> None:
         while True:
             text_message, message_type = await self.log_queue.get()
             await self._send_message_async(text_message, message_type)
             self.log_queue.task_done()
 
-    def send_message(self, text_message: str, message_type: int = None):
+    def send_message(self, text_message: str, message_type: int = None) -> None:
         """Send message to Google Chat, either synchronously or asynchronously."""
         self._ensure_queue_processing()
         try:
@@ -161,7 +187,7 @@ class GoogleChatAsyncMessenger(metaclass=SingletonMeta):
             asyncio.set_event_loop(loop)
             loop.run_until_complete(self._enqueue_message(text_message, message_type))
 
-    async def _send_message_async(self, text_message: str, message_type: int):
+    async def _send_message_async(self, text_message: str, message_type: int) -> None:
         """Send message to Google Chat asynchronously."""
         try:
             message_headers = {'Content-Type': 'application/json; charset=UTF-8'}
@@ -172,6 +198,10 @@ class GoogleChatAsyncMessenger(metaclass=SingletonMeta):
 
             # Determine color based on message type
             color = self.color_dict.get(message_type, '#000000')
+            if message_type == 0:
+                url = self.g_chat_error_webhook
+            else:
+                url = self.g_chat_webhook
 
             bot_message = {
                 "cards": [
@@ -197,23 +227,30 @@ class GoogleChatAsyncMessenger(metaclass=SingletonMeta):
 
             async with aiohttp.ClientSession() as session:
                 async with session.post(
-                        url=self.g_chat_webhook,
+                        url=url,
                         headers=message_headers,
                         data=json.dumps(bot_message),
                 ) as response:
                     response.raise_for_status()  # Raise an HTTPError if the HTTP request returned an unsuccessful status code
         except Exception as e:
-            self.logger.error('Error in webhookApi', exc_info=True)
+            if self.logger:
+                self.logger.error('Error in webhookApi', exc_info=True)
 
 
-def setup_google_chat_messenger(service_name: str, g_chat_webhook: str, environ: str, async_mode: bool = False,
-                                log_enabled: Optional[list] = None, logger: Optional[logging.Logger] = None) -> Union[
-    GoogleChatSyncMessenger, GoogleChatAsyncMessenger]:
+def setup_google_chat_messenger(
+        service_name: str,
+        g_chat_webhook: str,
+        g_chat_error_webhook: str,
+        environ: str,
+        async_mode: bool = False,
+        log_enabled: Optional[list[str]] = None,
+        logger: Optional[logging.Logger] = None
+) -> Union[GoogleChatSyncMessenger, GoogleChatAsyncMessenger]:
     if async_mode:
-        return GoogleChatAsyncMessenger(service_name, g_chat_webhook, environ, log_enabled, logger)
+        return GoogleChatAsyncMessenger(service_name, g_chat_webhook, g_chat_error_webhook, environ, log_enabled,
+                                        logger)
     else:
-        return GoogleChatSyncMessenger(service_name, g_chat_webhook, environ, log_enabled, logger)
-
+        return GoogleChatSyncMessenger(service_name, g_chat_webhook, g_chat_error_webhook, environ, log_enabled, logger)
 
 # async def main():
 #     async_messenger = GoogleChatAsyncMessenger(
